function [energy_eV,norm_xas,mu]=mback(Z,fname,range,Nth,core);
%MBACK	Normalize raw data to tabulated mass absorption coefficients
%		with erfc() and polynomials as the background functions.
%
%	[e, Xabs, mu] = mback(Z, 'fname', range, Nth, 'core','method')
%		
%		Z=atomic number or 'Z' as atomic symbol
%		'fname': data file containing [eV, raw] in ASCII format
%		range=[pre_edge1, pre_edge2, post_edge1, post_edge2].
%		Nth: order of the polynomials (default=2).
%		'core': specify 'K', 'L1', 'L2', 'L3', 'M1' edge
%				or choose automatically according to the energy range.
%
% Tsu-Chien Weng, James E. Penner-Hahn, 2000-06-27
% Copyleft (c), by the Penner-Hahn group, the University of Michigan

%2001-07-26: Implement the full non-linear optimization with Jacobian.


% (0) Checking input arguments
if nargin < 1, error('1st argument: Z or symbol.'), end
if nargin < 2, error('2nd argument: data file [eV, raw] in ASCII format.'), end
if nargin < 3 | isempty(range), range=[]; end
if nargin < 4 | isempty(Nth), Nth=2; end
if nargin < 5, core=''; end

if isstr(Z), Z=zymbol(Z); end
load element

% (1a) Reading raw data
eval(['load ' fname])
fname=strtok(fname,'.');
eval(['energy_eV=' fname '(:,1);'])	% energy in  eV
eval(['raw=' fname '(:,2);'])		% raw data: mu_obs
energy_KeV=energy_eV/1000;			% energy in KeV

% (1b) Finding the interested ranger of data points
edges=[
element(Z).K_edge, element(Z).K_alpha1;
element(Z).L_edge', repmat(element(Z).L_alpha1,3,1);
element(Z).M_edge, 0;
];									% in KeV
try
	switch upper(core)
		case 'K',	E_edge=edges(1,1); E_fluo=edges(1,2);
		case 'L1',	E_edge=edges(2,1); E_fluo=edges(2,2);
		case 'L2',	E_edge=edges(3,1); E_fluo=edges(3,2);
		case 'L3',	E_edge=edges(4,1); E_fluo=edges(4,2);
		case 'M1',	E_edge=edges(5,1); E_fluo=edges(5,2);
		case '',	edges=edges(edges(:,1) > energy_KeV(1) & edges(:,1) < energy_KeV(end),:); 
					E_edge=edges(1); E_fluo=edges(2);
		otherwise,	error([core '-shell edge has not been supported yet.'])
	end
catch
	error('Check the Atomic number/symbol or Core shell')
end
if isempty(range)
	pre_edge =E_edge - 0.020;		% skip XANES for proper scaling
	post_edge=E_edge + 0.080;		% default: [E_edge - 20eV, E_edge + 80eV]
	idx_pre =(energy_KeV <= pre_edge);
	idx_post=(energy_KeV >= post_edge);
	idx_xane=(energy_KeV > pre_edge & energy_KeV < post_edge);
elseif length(range) == 2
	pre_edge =E_edge - range(1)/1000;
	post_edge=E_edge + range(2)/1000;
	idx_pre =(energy_KeV <= pre_edge);
	idx_post=(energy_KeV >= post_edge);
	idx_xane=(energy_KeV >  pre_edge & energy_KeV < post_edge);
elseif length(range) == 4
	idx_pre =(energy_eV >= range(1) & energy_eV <= range(2));
	idx_post=(energy_eV >= range(3) & energy_eV <= range(4));
	idx_xane=(energy_eV >  range(2) & energy_eV <  range(3));
end
idx_fit=(idx_pre | idx_post);		% index for pre-/post-edge data points
num_pre =sum(idx_pre);
num_post=sum(idx_post);

% (2) Tabulated mass absorption coefficients
% Ref: McMaster et al. (1969) "Compilation of X-ray Cross sections"
mu=McMaster(Z,energy_KeV,'c');
idx_res=(energy_KeV < E_edge);
res=polyval(polyfit(energy_KeV(idx_res),mu(idx_res),2),energy_KeV);
mu=mu-res;
mu(idx_res)=0;

% (3a) Scale raw data and mu_tab
mu_span =max(mu(idx_xane));
raw_span=max(raw(idx_xane))-min(raw(idx_xane));
mu_tab= mu(idx_fit)/ mu_span;		% scaled McMaster data
mu_obs=raw(idx_fit)/raw_span;		% scaled raw data
KeV_obs=energy_KeV(idx_fit);

% (3b) Normalization
HndlFigBG=figure('NumberTitle','off','Name','Background');
plot(energy_KeV,raw/raw_span,'o','EraseMode', 'none');
hold on
HndlPlot=plot(KeV_obs,repmat(mu_obs,1,3),'r','EraseMode','xo');

xi0=0.5;
pi0=polyfit(KeV_obs(num_pre+1:end)-E_edge,mu_tab(num_pre+1:end),Nth);
GUESS=[   1,   -1, xi0,  pi0(1:Nth), 0];	% initial guess: [S,A,xi,{P_i}]
LOBND=[   0, -Inf,   0, -Inf*(1:Nth+1)];	% lower bounds
UPBND=[ Inf,    0, Inf,  Inf*(1:Nth+1)];	% upper bounds

weight_one=ones(size(raw));
weight_fit=[repmat(1/sqrt(num_pre),num_pre,1);repmat(1/sqrt(num_post),num_post,1)];
% weight_fit=ones(num_pre+num_post,1);

OPTS=optimset('LevenbergMarquardt','on','Jacobian','on','LargeScale','on','MaxFunEvals',2000);
fits=lsqcurvefit('mbgfun',GUESS,KeV_obs,mu_tab.*weight_fit,LOBND,UPBND,OPTS,mu_obs,E_edge,E_fluo,HndlPlot,weight_fit);
norm_xas=mbgfun(fits,energy_KeV,raw/raw_span,E_edge,E_fluo,HndlPlot,weight_one)*mu_span;

axis auto
hold off

% (4) Final graphical output
HndlFigNORM=figure('NumberTitle','off','Name','X-ray Absorbance');
plot(energy_eV,norm_xas,energy_eV,mu,'r:')
xlabel('Energy (eV)'), ylabel('X-ray Mass Attenuation Coefficients (cm^2/g)')

norm_mu=[energy_eV,norm_xas,mu];
eval(['save ' fname '.nrm norm_mu -ascii'])

return





function out=zymbol(in)
%ZYMBOL     Convert between atomic number and atomic symbol
%
%   Examples:   zymbol('Mn') gives 25.
%               zymbol(82)   gives 'Pb'.
%
% Tsu-Chien Weng, 2002-10-03
% Copyleft(c), the Penner-Hahn research group

row1='H He';
row2='LiBeB C N O F Ne';
row3='NaMgAlSiP S ClAr';
row4='K CaScTiV CrMnFeCoNiCuZnGaGeAsSeBrKr';
row5='RbSrY ZrNbMoTcRuRhPdAgCdInSnSbTeI Xe';
row6='CsBaLaCePrNdPmSmEuGdTbDyHoErTmYbLuHfTaW ReOsIrPtAuHgTlPbBiPoAtRn';
row7='FrRaAcThPaU NpPuAmCmBkCfEsFmMdNoLrRfDbSgBhHsMt';

periodicTable=[row1 row2 row3 row4 row5 row6 row7];
lastZ=length(periodicTable)/2;

if ischar(in)
    if length(in)==1, in=[in ' ']; end
    in=lower(in);
    in(1)=upper(in(1));
    atomicNumber=(findstr(in,periodicTable)-1)/2+1;
    out=atomicNumber;
elseif isnumeric(in)
    if in > lastZ, error('Check the atomic number.  It is too large.'), end
    atomicSymbol=periodicTable([2*in(:)-1 2*in(:)]);
    out=deblank(atomicSymbol);
else
    eval('help zymbol')
    error('Unrecognized argument format.')
end

return

